"use strict";
require("source-map-support/register");
var DependencyManager = (function () {
    function DependencyManager() {
        this.context = {};
    }
    DependencyManager.prototype.add = function (name, deps) {
        if (this.context[name])
            throw new Error("Duplicate dependency entry: " + name);
        if (deps && deps.length == 0)
            deps = null;
        this.context[name] = deps;
    };
    DependencyManager.prototype.resolveDependency = function (name, chain) {
        if (chain === void 0) { chain = []; }
        if (!this.context[name])
            return [];
        chain = chain.slice();
        if (chain.indexOf(name) >= 0) {
            chain.push(name);
            throw new Error("Recursive dependency detected: " + chain.join("\n=>"));
        }
        chain.push(name);
        var result = [];
        for (var _i = 0, _a = this.context[name]; _i < _a.length; _i++) {
            var dep = _a[_i];
            var deps = this.resolveDependency(dep, chain);
            for (var _b = 0, deps_1 = deps; _b < deps_1.length; _b++) {
                var subDep = deps_1[_b];
                if (result.indexOf(subDep) < 0)
                    result.push(subDep);
            }
            result.push(dep);
        }
        return result;
    };
    DependencyManager.prototype.resolve = function () {
        var result = [];
        for (var name_1 in this.context) {
            var deps = this.resolveDependency(name_1);
            for (var _i = 0, deps_2 = deps; _i < deps_2.length; _i++) {
                var dep = deps_2[_i];
                if (result.indexOf(dep) < 0)
                    result.push(dep);
            }
            if (result.indexOf(name_1) < 0)
                result.push(name_1);
        }
        return result;
    };
    return DependencyManager;
}());
exports.DependencyManager = DependencyManager;
//# sourceMappingURL=dependency.js.map